#!/bin/sh
#
# exportusers
#
# USAGE: see usage_()
#
#
# Last Updated     : $Author $
# Update Date Time : $Date $
# RCS File Name    : $RCSfile $
# RCS Revision     : $Revision $
# Symbolic name    : $Name $
# State            : $State $
#

. $HOME/etc/setenv.sh

#
# Prints usage information and exit
#
# [$1] error message
#
usage_()
{
	if [ -n "$1" ] ; then
		echo "$1" >&2
		echo >&2
	fi

	cat <<-EOFUSAGE
	  Usage:
	    exportusers [-g group]... [-f file] [-s|-S] [-x|-X] [-l|-L] [-p] [-u] [user1, user2, ...]

	        -g group        : export all users from group including group data (can be repeated)
	        -f file         : the export file name (default: eu.tar)
	        -s              : exclude system user settings (default if group or user list was specified)
	        -S              : include system user settings (default if no group or user list was specified) 
	        -x              : exclude support user settings (default if group or user list was specified)
	        -X              : include support user settings (default if no group or user list was specified) 
	        -l              : export location setting for each exported user account
			-L              : export all locations
			-p              : export plugin setting for each exported user account
			-u				: export group setting for each exported user account
	        user1, user2... : list of users or groups to be exported.
	                          User names starting with '#' will be treated as groups.
	                          In case of groups only the group data will be exported; to export users in the groups along with group data use the -g option
	                          (user names starting with '#' must be quoted otherwise shell will treat it as a comment)
	
	   Starting exportusers without any option/argument will export all users and groups
	EOFUSAGE

	[ -n "$1" ] && exit 1
	exit 0
}

# default output file
file="eu.tar"

# With no options all users should be exported
exportAll="true"

# for querying on groups or users
userWhereClause=""
groupWhereClause=""
userOrClause=""
groupOrClause=""

# Adding @system and @support settings
addSystem=""
addSupport=""

groupList=""
userList=""

# Adding location and plugig
exportAllLocation=""
exportPlugins=""
exportUserGroups=""

while getopts ':xXsSlLpuf:g:' OPT; do
	case $OPT in
		l)
			exportAllLocation="false"
			echo received -l ;;
		L)
			exportAllLocation="true"
			echo received -L ;;
		p)
			exportPlugins="true"
			echo received -p ;;
		u)
			exportUserGroups="true"
			echo received -u ;;
		s)
			addSystem="false" ;;
		S)
			addSystem="true" ;;
		x)
			addSupport="false" ;;
		X)
			addSupport="true" ;;
		g)
			exportAll="false"
			groupList="$groupList; #$OPTARG"
			userOrClause="$userOrClause OR AGROUP='$OPTARG'"
			groupOrClause="$groupOrClause OR ID='$OPTARG'" ;;
		f)
			file="$OPTARG" ;;
		*)
			usage_ "Invalid option: $OPT" ;;
	esac
done

shift $(($OPTIND-1))

userId=""
userId=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "select f.DBNAME FROM $FIELDSTBL as f inner join Dctables as tab on f.COLID=tab.COLID WHERE tab.TABLENAME ='Access' AND tab.COLID = 'USID';")
echo userId =$userId

if [ -n "$1" -o -n "$groupList" ] ; then
	# groups or users was specified
	[ -z "$addSystem" ] && addSystem="false"
	[ -z "$addSupport" ] && addSupport="false"
else
	# export all users
	[ -z "$addSystem" ] && addSystem="true"
	[ -z "$addSupport" ] && addSupport="true"
fi

while [ -n "$1" ] ; do
	exportAll="false"
	
	[ "$1" = "@system" ] && addSystem="true"
	[ "$1" = "@support" ] && addSupport="true"


	group="${1#\#}"
	if [ "$group" != "$1" ] ; then
		groupOrClause="$groupOrClause OR ID='$group'"
	else
		userOrClause="$userOrClause OR $userId='$1'"
	fi
	
	userList="$userList; $1"
	shift
done


[ -n "$userOrClause" ] && userWhereClause="WHERE ${userOrClause# OR }"
[ -n "$groupOrClause" ] && groupWhereClause="WHERE ${groupOrClause# OR }"

Users=""
Groups=""

if [ "$exportAll" = "true" -o -n "$userWhereClause" ] ; then
	Users=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "select $userId from $ACCESSTBL $userWhereClause;")
	[ "$addSystem" = "true" ] && Users="$Users @system"
	[ "$addSupport" = "true" ] && Users="$Users @support"
fi

if [ "$exportAll" = "true" -o -n "$groupWhereClause" ] ; then
	Groups=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "select ID from $GROUPSTBL $groupWhereClause;")
fi


echo Exporting users:
echo $Users
echo 

echo Exporting groups:
echo $Groups
echo 

mkdir $TMP/eu.$$
touch $TMP/eu.$$/eu.dmp

isLocationOk="true"
isPluginPerUserOk="true"
isUserGroupOk="true"

if [ -z "$LOCATIONTBL" -o -z "$USERLOCATIONTBL" ] ; then
	isLocationOk="false"
	echo "isLocationOk=$isLocationOk"
else
	resultLocation=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "show tables like '$LOCATIONTBL';")
	resultUserLocation=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "show tables like '$USERLOCATIONTBL';")
	[ -z "$resultLocation" -o -z "$resultUserLocation" ] && isLocationOk="false"
	echo "isLocationOk=$isLocationOk"
fi

if [ -z "$PLUGINPERUSERTBL" ] ; then
	isPluginPerUserOk="false"
	echo "isPluginPerUserOk=$isPluginPerUserOk"
else
	resultPluginPerUser=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "show tables like '$PLUGINPERUSERTBL';")
	[ -z "$resultPluginPerUser" ] && isPluginPerUserOk="false"
	echo "isPluginPerUserOk=$isPluginPerUserOk"
fi

if [ -z "$USERGROUPTBL" ] ; then
	isUserGroupOk="false"
	echo "isUserGroupOk=$isUserGroupOk"
else
	resultUserGroup=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "show tables like '$USERGROUPTBL';")
	[ -z "$resultUserGroup" ] && isUserGroupOk="false"
	echo "isUserGroupOk=$isUserGroupOk"
fi

if [ -n "$exportAllLocation" -a "$isLocationOk" = "true" ] ; then
	echo exportAllLocation= $exportAllLocation
	echo exportAll= $exportAll
	if [ $exportAllLocation = "true" ] ; then 
		if [ $exportAll = "true" ] ; then
			echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $LOCATIONTBL $USERLOCATIONTBL' -t >> '$TMP/eu.$$/eu.dmp
			$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $LOCATIONTBL $USERLOCATIONTBL -t >> $TMP/eu.$$/eu.dmp
		else
			echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $LOCATIONTBL' -t >> '$TMP/eu.$$/eu.dmp
			$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $LOCATIONTBL -t >> $TMP/eu.$$/eu.dmp
			userLocationOrClause=""
			for u in $Users; do
				userLocationOrClause="$userLocationOrClause OR $userId='$u'"
			done
			echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $USERLOCATIONTBL' -t -w '"${userLocationOrClause# OR }"' >> '$TMP/eu.$$/eu.dmp
			echo 'userLocationOrClause: ' $userLocationOrClause
			$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $USERLOCATIONTBL -t -w "${userLocationOrClause# OR }" >> $TMP/eu.$$/eu.dmp
		fi
	else
		Locations=""
		userLocationOrClause=""
		locationOrClause=""
		userLocationWhereClause=""
		for u in $Users; do
			userLocationOrClause="$userLocationOrClause OR $userId='$u'"
		done
		[ -n "$userLocationOrClause" ] && userLocationWhereClause="WHERE ${userLocationOrClause# OR }"
		Locations=$($MYSQL_HOME/bin/mysql -N $DBNAME <<< "select LOCATIONID from $USERLOCATIONTBL $userLocationWhereClause;")
		echo Locations: $Locations
		
		for l in $Locations; do
			substring="LOCATIONID='$l'"
			if [ "${locationOrClause#*$substring}" != "$locationOrClause" ] ; then
				echo "LOCATIONID='$l'" 'is in' \"$locationOrClause\" string.
				continue
			fi
			locationOrClause="$locationOrClause OR LOCATIONID='$l'"
		done
		echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $LOCATIONTBL' -t -w '"${locationOrClause# OR }"' >> '$TMP/eu.$$/eu.dmp
		$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $LOCATIONTBL -t -w "${locationOrClause# OR }" >> $TMP/eu.$$/eu.dmp
		echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $USERLOCATIONTBL' -t -w '"${userLocationOrClause# OR }"' >> '$TMP/eu.$$/eu.dmp
		$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $USERLOCATIONTBL -t -w "${userLocationOrClause# OR }" >> $TMP/eu.$$/eu.dmp
	fi
else
	message="The -l and the -L parameter is useless for this server."
	[ -z "$exportAllLocation" ] && message="exportAllLocation\'s length is zero"
	echo $message
fi	

#PluginPerUser export
if [ -n "$exportPlugins" -a "$exportPlugins" = "true" -a "$isPluginPerUserOk" = "true" ] ; then
	pluginPerUserOrClause=""
	for u in $Users; do
		pluginPerUserOrClause="$pluginPerUserOrClause OR $userId='$u'"
	done

	echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $PLUGINPERUSERTBL' -t -w '"${pluginPerUserOrClause# OR }"' >> '$TMP/eu.$$/eu.dmp
	$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $PLUGINPERUSERTBL -t -w "${pluginPerUserOrClause# OR }" >> $TMP/eu.$$/eu.dmp
else
	message="exportPlugins\'s length is zero or exportPlugins\'s value is not true."
	[ -n "$exportPlugins" -a "$exportPlugins" = "true" ] && message="The -p parameter is useless for this server."
	echo $message
fi

#UserGroup export
if [ -n "$exportUserGroups" -a "$exportUserGroups" = "true" -a "$isUserGroupOk" = "true" ] ; then
	userGroupOrClause=""
	for u in $Users; do
		userGroupOrClause="$userGroupOrClause OR $userId='$u'"
	done
	
	echo $MYSQL_HOME'/bin/mysqldump --disable-extended-insert -c '$DBNAME $USERGROUPTBL' -t -w '"${userGroupOrClause# OR }"' >> '$TMP/eu.$$/eu.dmp
	$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $USERGROUPTBL -t -w "${userGroupOrClause# OR }" >> $TMP/eu.$$/eu.dmp
else
	message="exportUserGroups\'s length is zero or exportUserGroups\'s value is not true."
	[ -n "$exportUserGroups" -a "$exportUserGroups" = "true" ] && message="The -p parameter is useless for this server."
	echo $message
fi

if [ -n "$userWhereClause" ] ; then
	$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $ACCESSTBL -t -w "${userWhereClause#WHERE }" >> $TMP/eu.$$/eu.dmp
elif [ "$exportAll" = "true" ] ; then
	$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $ACCESSTBL -t >> $TMP/eu.$$/eu.dmp
fi

if [ -n "$groupWhereClause" ] ; then
	$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $GROUPSTBL -t -w "${groupWhereClause#WHERE }" >> $TMP/eu.$$/eu.dmp
elif [ "$exportAll" = "true" ] ; then
	$MYSQL_HOME/bin/mysqldump --disable-extended-insert -c $DBNAME $GROUPSTBL -t >> $TMP/eu.$$/eu.dmp
fi


for u in $Users; do
	[ ! -e "${USERS_DATA}/$u/filters" -a -d "${FILTER_DATA}/$u" ] && mv "${USERS_DATA}/$u/filters" "${FILTER_DATA}/$u"
	cp -Rap ${USERS_DATA}/$u $TMP/eu.$$/ 2>/dev/null
done

for g in $Groups; do
	cp -Rap "${USERS_DATA}/#$g" $TMP/eu.$$/ 2>/dev/null
done


[ -f "$CONF/serverid.cfg" ] && . "$CONF/serverid.cfg"

cat <<-EOFINFO > $TMP/eu.$$/.info
	ServerId="${SELF_SERVER_ID}"
	ServerHostName="$(hostname)"
	DateTime="$(date +'%Y%m%d %H%M%S')"
	SystemIncluded="$addSystem"
	SupportIncluded="$addSupport"
EOFINFO

[ -n "$groupList" ] && echo "GroupList=\"${groupList#; }\"" >> $TMP/eu.$$/.info
[ -n "$userList" ] && echo "UserList=\"${userList#; }\"" >> $TMP/eu.$$/.info

tar cf "$file" -C "$TMP/eu.$$/" .

cat $TMP/eu.$$/.info


rm -rf $TMP/eu.$$
